@@ -268,6 +268,23 @@ def process_members(resume_from: int):
     exporter.generate_report(resume_from)
 
 
+@click.command()
+@click.option(
+    "--limit", default=0, help="Run a report on a partial subset of data"
+)
+@transaction.atomic()
+def overall_report(limit: int = 0):
+    """Process each member"""
+    import exporter
+
+    if limit > 0:
+        logging.info("Running a limited report on {} members".format(limit))
+
+    import json
+
+    print(json.dumps(exporter.overall_report(members=None, limit=limit)))
+
+
 @click.command()
 @click.argument("member_id")
 @transaction.atomic()
@@ -408,6 +425,14 @@ def random_samples(archive_name, count, only_crossref):
         logging.info("{} does not contain: {}".format(archive_name, entry))
 
 
+@click.command()
+def draw_graphs():
+    import exporter
+
+    # exporter.draw_graphs()
+    pass
+
+
 @click.command()
 def import_all():
     """Download and import all data (excluding HathiTrust)"""
@@ -429,6 +454,14 @@ def import_all():
     )
 
 
+@click.command()
+def rescore():
+    """Rescore all data"""
+    import exporter
+
+    exporter.rescore_all()
+
+
 @click.command()
 @click.argument("doi")
 @click.option(
@@ -467,6 +500,7 @@ def show_preservation(doi, literal=False, no_issn=False):
 
 if __name__ == "__main__":
     cli.add_command(clear_cache)
+    cli.add_command(draw_graphs)
     cli.add_command(import_all)
     cli.add_command(import_cariniana)
     cli.add_command(import_clockss)
@@ -478,9 +512,11 @@ if __name__ == "__main__":
     cli.add_command(import_ocul)
     cli.add_command(import_pkp)
     cli.add_command(import_portico)
+    cli.add_command(overall_report)
     cli.add_command(process_member)
     cli.add_command(process_members)
     cli.add_command(random_samples)
+    cli.add_command(rescore)
     cli.add_command(show_archives)
     cli.add_command(show_cache)
     cli.add_command(show_issn)
