@@ -1,10 +1,35 @@
 import json
 import logging
+import os
 from datetime import date
+from datetime import timedelta
 from typing import DefaultDict, Any
 
+from crossref.restful import MaxOffsetError
 
-def get_members(s3client, samples_bucket, samples_path) -> list[str]:
+SAMPLES_BUCKET = "samples-crossref-research"
+SAMPLES_PATH = "members-works/sample-2023-01-21/works/"
+
+ANNOTATION_BUCKET = "outputs.research.crossref.org"
+ANNOTATION_PATH = "annotations"
+ANNOTATION_FILENAME = "preservation.json"
+
+REPORT_BUCKET = "outputs.research.crossref.org"
+REPORT_PATH = "reports"
+REPORT_FILENAME = "preservation.json"
+
+CODE_BUCKET = "airflow-crossref-research-annotation"
+
+PARALLEL_JOBS = 5
+
+import streamlit as st
+import boto3
+
+
+@st.cache_data(ttl=3600)
+def get_members(
+    s3client=None, samples_bucket=None, samples_path=None
+) -> list[str]:
     """
     Retrieves the list of members from the S3 bucket
     :param s3client: the s3client object to use to fetch
@@ -20,6 +45,9 @@ def get_members(s3client, samples_bucket, samples_path) -> list[str]:
     # a bug in the sampling framework that leads to an unavailable
     # object)
 
+    if not s3client:
+        s3client = boto3.client("s3")
+
     import re
 
     r = re.compile(r"member-(\d+).jsonl")
@@ -37,6 +65,44 @@ def get_members(s3client, samples_bucket, samples_path) -> list[str]:
     ]
 
 
+@st.cache_data(ttl=timedelta(hours=24))
+def get_all_members_api():
+    """
+    Retrieves the list of members from the API so we can assign names
+    :return: a dictionary of member information
+    """
+
+    from crossref.restful import Etiquette, Members
+
+    my_etiquette = Etiquette(
+        "Preservation Status", "0.01", "https://eve.gd", "meve@crossref.org"
+    )
+
+    plus_token = st.secrets.get("CROSSREF_PLUS_TOKEN")
+
+    if plus_token:
+        members = Members(
+            etiquette=my_etiquette,
+            crossref_plus_token=plus_token,
+        )
+    else:
+        members = Members(
+            etiquette=my_etiquette,
+        )
+
+    members = members.all()
+
+    output = {}
+
+    try:
+        for member in members:
+            output[member["id"]] = member["primary-name"]
+    except MaxOffsetError:
+        pass
+
+    return output
+
+
 def list_bucket(s3client, samples_bucket, samples_path) -> list[str]:
     """
     Lists the contents of the samples bucket
@@ -112,6 +178,27 @@ def preservation_status(result) -> (dict, str):
     )
 
 
+def score(one_archive, two_archives, three_archives, sample_count):
+    if sample_count > 0:
+        # 25% in 1 or more archives
+        if (one_archive / sample_count) + (two_archives / sample_count) + (
+            three_archives / sample_count
+        ) >= 0.25:
+            return "Bronze"
+
+        # 50% in 2 or more archives
+        if (two_archives / sample_count) + (
+            three_archives / sample_count
+        ) >= 0.50:
+            return "Silver"
+
+        # 75% in 3 archives
+        if three_archives / sample_count >= 0.75:
+            return "Gold"
+
+    return "Unclassified"
+
+
 def process_member_sample(samples, sample_path, verbose=False) -> dict:
     """
     Processes samples for a single member
@@ -251,20 +338,12 @@ def process_member_sample(samples, sample_path, verbose=False) -> dict:
     overall_status["preserved-in-three-or-more-archives"] = three_archives
 
     # determine the classes
-    overall_status["member-grade"] = "Unclassified"
-
-    if overall_status["sample-count"] > 0:
-        # 25% in 1 archive
-        if one_archive / overall_status["sample-count"] >= 0.25:
-            overall_status["member-grade"] = "Bronze"
-
-        # 50% in 2 archives
-        if two_archives / overall_status["sample-count"] >= 0.50:
-            overall_status["member-grade"] = "Silver"
-
-        # 75% in 3 archives
-        if three_archives / overall_status["sample-count"] >= 0.75:
-            overall_status["member-grade"] = "Gold"
+    overall_status["member-grade"] = score(
+        one_archive,
+        two_archives,
+        three_archives,
+        overall_status["sample-count"],
+    )
 
     return overall_status
 
@@ -272,28 +351,23 @@ def process_member_sample(samples, sample_path, verbose=False) -> dict:
 def push_json_to_s3(
     s3,
     json_obj,
-    member_id,
-    annotation_bucket,
-    annotation_path,
-    annotation_filename,
+    bucket,
+    path,
 ) -> None:
     """
     Writes the JSON data to S3
     :param s3: the s3 object to use
-    :param annotation_bucket: the name of the annotation bucket
-    :param annotation_path: the path of the annotation object
-    :param annotation_filename: the name of the annotation file
+    :param bucket: the name of the bucket
+    :param path: the path of the object to store
     :param json_obj: the JSON to write
-    :param member_id: the member ID into which to write
     :return:
     """
     import json
     import logging
 
-    logging.info("Writing JSON to S3")
-    key = f"{annotation_path}/members/{member_id}/{annotation_filename}"
+    logging.info(f"Writing JSON to S3 bucket {bucket} at path {path}")
 
-    obj = s3.Object(annotation_bucket, key)
+    obj = s3.Object(bucket, path)
     obj.put(Body=json.dumps(json_obj))
 
 
@@ -344,30 +418,19 @@ def process_sample(
         samples, samples_path, verbose=verbose
     )
 
+    key = f"{annotation_path}/members/{member_id}/{annotation_filename}"
+
     push_json_to_s3(
-        s3,
-        overall_status,
-        member_id,
-        annotation_bucket=annotation_bucket,
-        annotation_path=annotation_path,
-        annotation_filename=annotation_filename,
+        s3=s3,
+        json_obj=overall_status,
+        bucket=annotation_bucket,
+        path=key,
     )
 
     return {member_id: overall_status}
 
 
 def generate_report(resume_from: int):
-    SAMPLES_BUCKET = "samples-crossref-research"
-    SAMPLES_PATH = "members-works/sample-2023-01-21/works/"
-
-    ANNOTATION_BUCKET = "outputs.research.crossref.org"
-    ANNOTATION_PATH = "annotations"
-    ANNOTATION_FILENAME = "preservation.json"
-
-    CODE_BUCKET = "airflow-crossref-research-annotation"
-
-    PARALLEL_JOBS = 5
-
     import boto3
     import logging
 
@@ -411,22 +474,77 @@ def generate_report(resume_from: int):
     return members
 
 
-def overall_report(members):
-    SAMPLES_BUCKET = "samples-crossref-research"
-    SAMPLES_PATH = "members-works/sample-2023-01-21/works/"
-
-    ANNOTATION_BUCKET = "outputs.research.crossref.org"
-    ANNOTATION_PATH = "annotations"
-    ANNOTATION_FILENAME = "preservation.json"
-
-    CODE_BUCKET = "airflow-crossref-research-annotation"
-
-    PARALLEL_JOBS = 5
-
+def rescore_member(
+    annotation_bucket,
+    annotation_filename,
+    annotation_path,
+    samples_bucket,
+    samples_path,
+    member_id,
+    code_bucket,
+    verbose=False,
+):
     import boto3
     import logging
 
     s3client = boto3.client("s3")
+    session = boto3.Session()
+    s3 = session.resource("s3")
+
+    preservations = get_annotation(
+        s3client=s3client,
+        annotations_bucket=ANNOTATION_BUCKET,
+        annotations_path=ANNOTATION_PATH,
+        member_id=member_id,
+        annotation_name="preservation.json",
+    )
+
+    one_archive = preservations["preserved-in-one-archive"]
+    two_archives = preservations["preserved-in-two-archives"]
+    three_archives = preservations["preserved-in-three-or-more-archives"]
+
+    sample_count = preservations["sample-count"]
+
+    # determine the classes
+    preservations["member-grade"] = score(
+        one_archive,
+        two_archives,
+        three_archives,
+        preservations["sample-count"],
+    )
+
+    key = f"{annotation_path}/members/{member_id}/{annotation_filename}"
+
+    logging.info(
+        f"Rescoring member {member_id} as {preservations['member-grade']}"
+    )
+
+    push_json_to_s3(
+        s3=s3,
+        json_obj=preservations,
+        bucket=annotation_bucket,
+        path=key,
+    )
+
+
+def overall_report(members=None, limit=0):
+    import boto3
+    import logging
+
+    s3client = boto3.client("s3")
+    session = boto3.Session()
+    s3 = session.resource("s3")
+
+    if not members:
+        # get member list from S3
+        members = get_members(
+            s3client=s3client,
+            samples_bucket=SAMPLES_BUCKET,
+            samples_path=SAMPLES_PATH,
+        )
+
+    if limit > 0:
+        members = members[:limit]
 
     # overall reports we want to build:
     # 1. Breakdown by member size
@@ -465,6 +583,20 @@ def overall_report(members):
     unclassified_members = {member_tier: 0 for member_tier in member_tier_names}
 
     for member in members:
+        logging.info(f"Fetching preservation statistics for member {member}")
+
+        from crossref.restful import Etiquette, Members
+
+        my_etiquette = Etiquette(
+            "Preservation Status", "0.01", "https://eve.gd", "meve@crossref.org"
+        )
+
+        members = Members(etiquette=my_etiquette)
+        member_obj = members.member(member)
+
+        if "primary-name" not in member_obj:
+            print(member_obj)
+
         preservations = get_annotation(
             s3client=s3client,
             annotations_bucket=ANNOTATION_BUCKET,
@@ -484,10 +616,19 @@ def overall_report(members):
         annual_fee = int(member_data["annual-fee"])
         member_band = None
 
+        logging.info(
+            f"{member_obj['primary-name']} has an annual fee of {annual_fee}"
+        )
+
         for key, val in member_tiers.items():
-            if annual_fee <= val:
+            if annual_fee <= int(val):
                 # put this member in this band
+                logging.info(f"{member_obj['primary-name']} is in band {key}")
                 member_band = key
+                break
+
+            if member_band:
+                break
 
         if not member_band:
             logging.warning(f"Unable to classify this member ({member})")
@@ -495,12 +636,16 @@ def overall_report(members):
         # this calculates the preservation grades for different member tiers
         # using fee level as the benchmark
         if preservations["member-grade"] == "Bronze":
+            logging.info(f"{member_obj['primary-name']} is Bronze")
             bronze_members[member_band] += 1
         elif preservations["member-grade"] == "Silver":
+            logging.info(f"{member_obj['primary-name']} is Silver")
             silver_members[member_band] += 1
         elif preservations["member-grade"] == "Gold":
+            logging.info(f"{member_obj['primary-name']} is Gold")
             gold_members[member_band] += 1
         else:
+            logging.info(f"{member_obj['primary-name']} is Unclassified")
             unclassified_members[member_band] += 1
 
         pass
@@ -510,13 +655,74 @@ def overall_report(members):
     bronze_totals = sum(bronze_members.values())
     unclassified_totals = sum(unclassified_members.values())
 
-    return
+    # bundle this all up
+    overall_result = {
+        "gold-members": gold_members,
+        "silver-members": silver_members,
+        "bronze-members": bronze_members,
+        "unclassified-members": unclassified_members,
+        "gold-totals": gold_totals,
+        "silver-totals": silver_totals,
+        "bronze-totals": bronze_totals,
+        "unclassified-totals": unclassified_totals,
+    }
+
+    key = f"{REPORT_PATH}/{REPORT_FILENAME}"
+
+    push_json_to_s3(
+        s3=s3,
+        json_obj=overall_result,
+        bucket=REPORT_BUCKET,
+        path=key,
+    )
+
+    return overall_result
 
 
+def rescore_all(members=None, limit=0):
+    import boto3
+    import logging
+
+    s3client = boto3.client("s3")
+    session = boto3.Session()
+    s3 = session.resource("s3")
+
+    if not members:
+        # get member list from S3
+        members = get_members(
+            s3client=s3client,
+            samples_bucket=SAMPLES_BUCKET,
+            samples_path=SAMPLES_PATH,
+        )
+
+    if limit > 0:
+        members = members[:limit]
+
+    from joblib import Parallel, delayed
+
+    for member in members:
+        results = Parallel(n_jobs=PARALLEL_JOBS)(
+            delayed(rescore_member)(
+                ANNOTATION_BUCKET,
+                ANNOTATION_FILENAME,
+                ANNOTATION_PATH,
+                SAMPLES_BUCKET,
+                SAMPLES_PATH,
+                member_id,
+                CODE_BUCKET,
+            )
+            for member_id in members
+        )
+
+
+@st.cache_data(ttl=timedelta(hours=24))
 def get_annotation(
     s3client, member_id, annotations_bucket, annotations_path, annotation_name
 ):
-    key = f"{annotations_path}/members/{member_id}/{annotation_name}.jsonl"
+    if s3client is None:
+        s3client = boto3.client("s3")
+
+    key = f"{annotations_path}/members/{member_id}/{annotation_name}"
 
     data = (
         s3client.get_object(Bucket=annotations_bucket, Key=key)["Body"]
@@ -530,4 +736,5 @@ def get_annotation(
 
     with StringIO(data) as json_file:
         output = json.loads(json_file.read())
+
         return output
